!------------------------------------------------------------------------------|
!------------------------------------------------------------------------------|
!                                                                              |
!              ||===\\                                                         | 
!              ||    \\                                                        |
!              ||     ||   //==\\   ||  ||   //==||  ||/==\\                   |
!              ||     ||  ||    ||  ||  ||  ||   ||  ||    ||                  |
!              ||    //   ||    ||  ||  ||  ||   ||  ||                        |
!              ||===//     \\==//    \\==\\  \\==\\  ||                        |
!                                                                              |
!------------------------------------------------------------------------------|
!------------------------------------------------------------------------------|
!                                                                              |
!              DEFINE_BC_ROT_SUB_INIT   Feb. 2009                              |
!                                                                              |
!------------------------------------------------------------------------------|
!------------------------------------------------------------------------------|

subroutine define_bc_rot_sub_init(params,osolve,vo,bcdef)

!------------------------------------------------------------------------------|
!((((((((((((((((  Purpose of the routine  )))))))))))))))))))))))))))))))))))))
!------------------------------------------------------------------------------|
! This routine assigns the velocity boundary conditions for the rotational     |
! subduction geometry                                                          |
!------------------------------------------------------------------------------|
!((((((((((((((((  declaration of the subroutine arguments  ))))))))))))))))))))
!------------------------------------------------------------------------------|

use definitions
!use mpi

implicit none

include 'mpif.h' 

type (parameters) params
type (octreesolve) osolve
type (void) vo
type (bc_definition) bcdef

!------------------------------------------------------------------------------|
!(((((((((((((((( declaration of the subroutine internal variables )))))))))))))
!------------------------------------------------------------------------------|

integer :: i,iproc,nproc,ierr
integer :: tempcase
double precision :: eps,pi,x0,y0,h,rx,ryn,rys,vin
double precision :: x1,y1,ry
double precision :: rmain,zmain,omega,theta,distmain,distrot
double precision :: z0,shellwidth,relaxwidth
double precision :: xshell,xedge,distz0plane,tstart,tend,tstart2,tend2,trans
double precision :: geotherm

!------------------------------------------------------------------------------|
!------------------------------------------------------------------------------|

call mpi_comm_size (mpi_comm_world,nproc,ierr)
call mpi_comm_rank (mpi_comm_world,iproc,ierr)

!basic constants
eps=1.d-10
pi=atan(1.d0)*4.d0

!set fixed to 0 everywhere
osolve%kfix=0
osolve%kfixt=0


!parameters from input file 
x0=bcdef%bc_parameters(1)
y0=bcdef%bc_parameters(2)
h=bcdef%bc_parameters(3)
rx=bcdef%bc_parameters(4)
ryn=bcdef%bc_parameters(5)
rys=bcdef%bc_parameters(6)
vin=bcdef%bc_parameters(7)
shellwidth = bcdef%bc_parameters(8)    
tempcase=idint(bcdef%bc_parameters(9))
!nl_heat=bcdef%bc_parameters(10)/params%tempscale
!hr=bcdef%bc_parameters(11)
z0=bcdef%bc_parameters(12)
relaxwidth = bcdef%bc_parameters(13)

!Derived parameters from input values
rmain=(h**2+rx**2)/(2.d0*h)
zmain=h-rmain
omega=vin/rmain

!rotating element is sphere

do i=1,osolve%nnode

!cylindrical tube beind x0
if (osolve%x(i)<x0) then
    x1 = x0
else
    x1 = osolve%x(i);
end if

!set y radius for northern or southern part, respecitvely
if (osolve%y(i)>y0) then
    ry = ryn
else
    ry = rys
end if

!if width is negative, shape is cylinder up to edge
if (ry <= 0.d0 ) then
    y1 = y0
    ry = rx
else
    y1 = osolve%y(i)
end if

!distance from center of sphere
distmain=sqrt((x1-x0)**2+((y1-y0)*rx/ry)**2 + &
              (osolve%z(i)*params%vex-zmain)**2)
!distance from axis of rotation
distrot=sqrt((x1-x0)**2+(osolve%z(i)*params%vex-zmain)**2)

!setting velocity in indenter
if (distmain<rmain) then
    theta=asin(x1/distrot)
    osolve%kfix((i-1)*3+1)=1
    osolve%u(i)=distrot*omega*cos(theta)
    osolve%kfix((i-1)*3+2)=1
    osolve%v(i)=0.d0
    osolve%kfix((i-1)*3+3)=1
    osolve%w(i)=-x1*omega
elseif (distmain > rmain+shellwidth .OR. &   !above spherical shell  
        osolve%z(i) > z0/params%vex - 2.d0**(-params%levelmax_oct) ) then !two nodes directly under topmost surface also fixed
    osolve%kfix((i-1)*3+1)=1
    osolve%u(i)=0.d0
    osolve%kfix((i-1)*3+2)=1
    osolve%v(i)=0.d0
    osolve%kfix((i-1)*3+3)=1
    osolve%w(i)=0.d0
end if

!y=0 face free slip 
if (osolve%y(i)<eps) then
    !no flux through face
    osolve%kfix((i-1)*3+2)=1 ; osolve%v(i)=0.d0
    !Fix temperature to initialisation solution
    if (tempcase==-1) osolve%kfixt(i)=1
end if

!y=1 face free slip
if (osolve%y(i)>(1.d0-eps)) then
    !no flux through face
    osolve%kfix((i-1)*3+2)=1 ; osolve%v(i)=0.d0
    !Fix temperature to initialisation solution
    if (tempcase==-1) osolve%kfixt(i)=1
end if

!x=0 face
if (osolve%x(i)<eps) then
    !set temperature
    osolve%kfixt(i) = 1
    if (tempcase .ne. -1) then
        osolve%temp(i) = geotherm (osolve%z(i)*params%vex,bcdef,params)
    end if
end if

!z=0 face
if (osolve%z(i) < eps) then
    if (rmain+shellwidth < distmain .AND. distmain < rmain+shellwidth+relaxwidth) then
        !min temperature at end of spherical shell is that of height h (assumption!) 
        tend = geotherm (h,bcdef,params)
        tstart = 1.d0

        !maxium extent of spherical shell in x direction
        xshell = sqrt((rmain+shellwidth)**2-zmain**2) 
        !x-position of radial intercept with outer shell edge
        distz0plane = sqrt(x1**2+((osolve%y(i)-y0)*rx/(ry))**2)
        xedge = x1*xshell/distz0plane 

        !calculating the start temperature along the sperical shell outline
        tstart2 = tstart + (tend-tstart)*x1/xshell
        
        !transition variable in relax-shell, goes from 0 to 1
        trans = (distmain-rmain-shellwidth)/relaxwidth
        !temperature at outer relax-shell width
        tend2 = 1.d0
        !calculate temperature in relaxation shell
        osolve%temp(i) = tstart2 + (tend2-tstart2)*trans
        osolve%kfixt(i) = 1

    elseif (distmain >= rmain+shellwidth+relaxwidth) then
        osolve%kfixt(i) = 1
        osolve%temp(i) = 1.d0
    end if      
end if

!x=1 face 
if (osolve%x(i)>1.d0-eps) then
    !Fix temperature to initialisation solution
    if (tempcase==-1) osolve%kfixt(i)=1
end if

!z=1 face 
if (osolve%z(i) > 1.d0-eps) then
    osolve%kfix((i-1)*3+1)=1 ; osolve%u(i)=0.d0 !x-direction
    osolve%kfix((i-1)*3+2)=1 ; osolve%v(i)=0.d0 !y-direction
    osolve%kfix((i-1)*3+3)=1 ; osolve%w(i)=0.d0 !z-direction
    osolve%kfixt(i)=1 ; osolve%temp(i)=0.d0
end if

if (.not.vo%influid(i)) then
   osolve%kfixt(i)=1
   osolve%temp(i)=0.d0
endif

end do

end

!------------------------------------------------------------------------------|
double precision function geotherm (z, bcdef, params)

    use definitions
    implicit none

    !input variables
    type (bc_definition) bcdef
    type (parameters) params
    double precision :: z

    !internal variables
    integer :: tempcase
    double precision :: depth, nl_heat, hr, z0


    tempcase=idint(bcdef%bc_parameters(9))
    nl_heat=bcdef%bc_parameters(10)/params%tempscale
    hr=bcdef%bc_parameters(11)
    z0=bcdef%bc_parameters(12)

    depth = z0 - z

    select case (tempcase)
        case (1)
        !"typical isotherm taken from R.Bendicks simulation
        !nl_heat is scaled nonlinear heat component hr^2*S_o/k_T
            geotherm = (depth/z0) + & 
                nl_heat*((1.d0-exp(-depth/hr))-(1.d0-exp(-z0/hr))*depth/z0)
        case default
            geotherm = depth/z0
    end select

end function geotherm
!------------------------------------------------------------------------------|
