!------------------------------------------------------------------------------|
!------------------------------------------------------------------------------|
!                                                                              |
!              ||===\\                                                         | 
!              ||    \\                                                        |
!              ||     ||   //==\\   ||  ||   //==||  ||/==\\                   |
!              ||     ||  ||    ||  ||  ||  ||   ||  ||    ||                  |
!              ||    //   ||    ||  ||  ||  ||   ||  ||                        |
!              ||===//     \\==//    \\==\\  \\==\\  ||                        |
!                                                                              |
!------------------------------------------------------------------------------|
!------------------------------------------------------------------------------|
!                                                                              |
!              DEFINE_BC_ROT_SUBDUCTION     AUG. 2014                          |
!                                                                              |
!------------------------------------------------------------------------------|
!------------------------------------------------------------------------------|

subroutine define_bc_rot_subduction(params,osolve,vo,bcdef)

!------------------------------------------------------------------------------|
!((((((((((((((((  Purpose of the routine  )))))))))))))))))))))))))))))))))))))
!------------------------------------------------------------------------------|
! This routine assigns the velocity boundary conditions for the rotational     |
! subduction geometry                                                          |
!------------------------------------------------------------------------------|
!((((((((((((((((  declaration of the subroutine arguments  ))))))))))))))))))))
!------------------------------------------------------------------------------|

use definitions
!use mpi

implicit none

include 'mpif.h' 

type (parameters) params
type (octreesolve) osolve
type (void) vo
type (bc_definition) bcdef

!------------------------------------------------------------------------------|
!(((((((((((((((( declaration of the subroutine internal variables )))))))))))))
!------------------------------------------------------------------------------|

integer i,iproc,nproc,ierr,tempcase,strictflow,indom
double precision :: eps,pi,x0,y0,h,rx,ryn,rys,vin,vtop,vback,nelemx,nelemz,dxy,dz
double precision :: x1,y1,ry
double precision :: rmain,zmain,omega,theta,distmain,distrot,distz0plane
double precision :: distzonerad,distzoney,distzone,distedge
double precision :: xedge,xminedge,yfix_xedge,yedge,yminedge
double precision :: zedge,zminedge,zmaxedge, ustart,ustart2,uend,uend2
double precision :: wstart,wstart2,wend,temp2,tempstart,tempstart2,tempend

!------------------------------------------------------------------------------|
!------------------------------------------------------------------------------|

call mpi_comm_size (mpi_comm_world,nproc,ierr)
call mpi_comm_rank (mpi_comm_world,iproc,ierr)

!basic constants
eps=1.d-10
pi=atan(1.d0)*4.d0

!set fixed to 0 everywhere
osolve%kfix=0
osolve%kfixt=0


!parameters from input file 
x0=bcdef%bc_parameters(1)
y0=bcdef%bc_parameters(2)
h=bcdef%bc_parameters(3)
rx=bcdef%bc_parameters(4)
ryn=bcdef%bc_parameters(5)
rys=bcdef%bc_parameters(6)
vin=bcdef%bc_parameters(7)
vtop=bcdef%bc_parameters(8)
vback=-bcdef%bc_parameters(9) !change in orientation
nelemx=bcdef%bc_parameters(10)
nelemz=bcdef%bc_parameters(11)
tempcase=idint(bcdef%bc_parameters(12))
temp2=bcdef%bc_parameters(13)/params%tempscale
strictflow=idint(bcdef%bc_parameters(14))
indom=idint(bcdef%bc_parameters(15))

!adjust bottom temperature case for consistency
if(indom==2 .or. indom==3) then
    tempcase = 2
endif

dxy=nelemx/2.d0**(params%levelmax_oct+1.d0) !0.5*transition width in xy
dz=nelemz*params%vex/2.d0**(params%levelmax_oct+1.d0) !0.5*transition width in z



!Derived parameters from input values
rmain=(h**2+rx**2)/(2.d0*h)
zmain=h-rmain
omega=vin/rmain


do i=1,osolve%nnode
    !cylindrical tube beind x0
    if (osolve%x(i)<x0) then
        x1 = x0
    else
        x1 = osolve%x(i);
    end if

    !set y radius for northern or southern part, respecitvely
    if (osolve%y(i)>y0) then
        ry = ryn
    else
        ry = rys
    end if
    
    !if width is negative, shape is cylinder up to edge
    if (ry <= 0.d0 ) then
        y1 = y0
        ry = rx
    else
        y1 = osolve%y(i)
    end if

    !distance from center of sphere
    distmain=sqrt((x1-x0)**2+((y1-y0)*rx/ry)**2 + &
                  (osolve%z(i)*params%vex-zmain)**2)
    !distance from axis of rotation
    distrot=sqrt((x1-x0)**2+(osolve%z(i)*params%vex-zmain)**2)
    
    !setting T and velocity in indenter
    if (distmain<rmain) then
        if (indom==1 .or. indom==3) then
        !set velocity
            theta=asin(x1/distrot)
            osolve%kfix((i-1)*3+1)=1
            osolve%u(i)=distrot*omega*cos(theta)
            osolve%kfix((i-1)*3+2)=1
            osolve%v(i)=0.d0
            osolve%kfix((i-1)*3+3)=1
            osolve%w(i)=-x1*omega
        end if
        if (indom==2 .or. indom==3) then
        !set temperature
            osolve%kfixt(i)=1 
            osolve%temp(i)=temp2
        end if
    end if


    !y=0 face free slip 
    if (osolve%y(i)<eps) then
        !no flux through face
        osolve%kfix((i-1)*3+2)=1 ; osolve%v(i)=0.d0

        !strictflow: flow fixed to (vback, 0 0)
        if (strictflow == 1) then
            osolve%kfix((i-1)*3+1)=1 ; osolve%u(i)=vback
            osolve%kfix((i-1)*3+3)=1 ; osolve%w(i)=0.d0
        end if
        
        !rotating sphere cuts y=0 face
        if (y0-ry < 0.d0) then
            !calculate geometry for face
            distedge = sqrt(max(0.0d0, rmain**2 - ((-y0)*rx/(ry))**2))
            if (strictflow == 1) then
                distzonerad = max(0.d0,min(1.d0,(distrot-distedge+dz)/(2*dz)))
            else
                if (distrot > distedge) then
                    distzonerad = 1.d0
                else
                    distzonerad = 0.d0
                end if
            end if
            
            !Setting velocities for inside and tranistion
            !angle of rotation
            theta=asin(x1/distrot)
            !inside rotating cylinder
            if (distzonerad < eps) then
                !set x velocity
                osolve%kfix((i-1)*3+1)=1
                osolve%u(i)=distrot*omega*cos(theta)
                !set z velocity
                osolve%kfix((i-1)*3+3)=1
                osolve%w(i)=-x1*omega
            !in transition zone, only occurs for strictflow = 1
            else if (distzonerad < 1.d0) then
                xminedge = x1*(distedge-dz)/distrot
                !set x velocity
                ustart=(distedge-dz)*omega*cos(theta)
                uend=vback
                osolve%u(i)=ustart+distzonerad*(uend-ustart)
                !set z velocity
                wstart=-xminedge*omega
                wend=0.d0
                osolve%w(i)=wstart+distzonerad*(wend-wstart)
            end if
        end if
    end if

    !y=1 face free slip
    if (osolve%y(i)>(1.d0-eps)) then
        !no flux through face
        osolve%kfix((i-1)*3+2)=1 ; osolve%v(i)=0.d0

        !strictflow: flow fixed to (vback, 0 0)
        if (strictflow == 1) then
            osolve%kfix((i-1)*3+1)=1 ; osolve%u(i)=vback
            osolve%kfix((i-1)*3+3)=1 ; osolve%w(i)=0.d0
        end if
        
        !rotating sphere cuts y=1 face
        if (y0+ry > 1.d0) then
            !calculate geometry for face
            distedge = sqrt(max(0.0d0, rmain**2 - ((1.d0-y0)*rx/(ry))**2))
            if (strictflow == 1) then
                distzonerad = max(0.d0,min(1.d0,(distrot-distedge+dz)/(2*dz)))
            else
                if (distrot > distedge) then
                    distzonerad = 1.d0
                else
                    distzonerad = 0.d0
                end if
            end if
            
            !Setting velocities for inside and transition
            !angle of rotation
            theta=asin(x1/distrot)
            !inside rotating cylinder
            if (distzonerad < eps) then
                !set x velocity
                osolve%kfix((i-1)*3+1)=1
                osolve%u(i)=distrot*omega*cos(theta)
                !set z velocity
                osolve%kfix((i-1)*3+3)=1
                osolve%w(i)=-x1*omega
            !in transition zone, only occurs for strictflow = 1
            else if (distzonerad < 1.d0) then
                xminedge = x1*(distedge-dz)/distrot
                !set x velocity
                ustart=(distedge-dz)*omega*cos(theta)
                uend=vback
                osolve%u(i)=ustart+distzonerad*(uend-ustart)
                !set z velocity
                wstart=-xminedge*omega
                wend=0.d0
                osolve%w(i)=wstart+distzonerad*(wend-wstart)
            end if
        end if
    end if
    
    !x=0 face
    if (osolve%x(i)<eps) then
        !set y and z velocity
        osolve%kfix((i-1)*3+1)=1 !x-direction fixed
        osolve%kfix((i-1)*3+2)=1 ; osolve%v(i)=0.d0 !y-direction
        osolve%kfix((i-1)*3+3)=1 ; osolve%w(i)=0.d0 !z-direction
        
        !outline of indenter
        zedge = max(0.d0, zmain+sqrt(max(0.d0,rmain**2-((y1-y0)*rx/(ry))**2)))
        zminedge = max(0.d0,zedge-dz)
        zmaxedge = max(0.d0,zedge+dz)
        
        !position relative to radial and y transition zones
        !    (0 inside, 1 outside, linear transition in zone)
        distzonerad = max(0.d0,min(1.d0,(osolve%z(i)*params%vex-zminedge)/(zmaxedge-zminedge+eps)))
        distzoney=min(1.d0,max(0.d0,(abs(y1-y0)-ry+dxy)/(2.d0*dxy)))
        
        !outside rotating sphere radially
        if (distzonerad > 1.d0-eps) then
            ustart = vtop
            uend = vback
            osolve%u(i)=ustart+distzoney*(uend-ustart)
        !within rotating sphere radially
        elseif (distzonerad < eps) then
            osolve%u(i)=omega*(osolve%z(i)*params%vex-zmain)
        !inside transition zone radially
        else
            !uend depends on upper gradient (see distzonerad > 1)
            ustart2=vtop
            uend2=vback
            uend=ustart2+distzoney*(uend2-ustart2)
            !ustart is gradient from urot to uback
            ustart2 = omega*(osolve%z(i)*params%vex-zmain)
            uend2=vback
            ustart=ustart2+distzoney*(uend2-ustart2)
            osolve%u(i)=ustart+distzonerad*(uend-ustart)
        end if
    end if
    
    !z=0 face
    if (osolve%z(i)*params%vex < eps) then
        
        !fix all velocities and temp, set v to 0
        osolve%kfix((i-1)*3+1)=1 
        osolve%kfix((i-1)*3+2)=1 ; osolve%v(i)=0.d0 !y-direction
        osolve%kfix((i-1)*3+3)=1
        osolve%kfixt(i)=1 
        
        !calculating radial transition zone: radial belt of width 2dxy at indenter edge,
        !circle origin is (x0, y0, 0)
        !finding intersections with indenter edge radial direction
        distz0plane = sqrt(x1**2+((y1-y0)*rx/(ry))**2)
        if (distz0plane < eps) then
            xedge = rx
            yedge = y0
        else
            xedge = x1*rx/distz0plane 
            yedge = y0+(y1-y0)*rx/distz0plane
        end if
        !finding intersection with indenter edge for given x in y direction
        yfix_xedge=sqrt(max(0.d0,rmain**2-((y1-y0)*rx/(ry))**2-zmain**2))
        !start of transition zone radially
        xminedge = xedge-dxy*xedge/sqrt(xedge**2+(yedge-y0)**2)
        yminedge = yedge-dxy*(yedge-y0)/sqrt(xedge**2+(yedge-y0)**2)

        !transition zone in radial >1 for outside, <0 for inside
        distzonerad=(sqrt((y1-y0)**2+x1**2) - &
                  sqrt((yminedge-y0)**2+(xminedge)**2))/(2.d0*dxy)

        !outside rotating sphere radially
        if (distzonerad > 1.d0) then
            !velocity
            osolve%u(i) = vback
            osolve%w(i) = 0.d0
            !temperature
            select case(tempcase)
                case (-1)
                    continue
                case (1)
                    osolve%temp(i) = temp2+x1*(1.0d0-temp2)
                case (2,3,4,5)
                    osolve%temp(i)=1.d0
                case default
                    osolve%temp(i)=1.d0
            end select
        !within rotating sphere radially
        elseif (distzonerad < 0.d0) then
            distrot = sqrt(x1**2+zmain**2)
            theta = asin(x1/distrot)
            !set x velocity
            osolve%u(i) = distrot*omega*cos(theta)
            !set z velocity
            osolve%w(i) = -x1*omega
            select case(tempcase)
                case (-1)
                    continue
                case (1)
                    osolve%temp(i) = temp2+x1*(1.0d0-temp2)
                case (2)
                    osolve%temp(i) = temp2
                case (3)
                    osolve%temp(i) = 1.d0+x1/rx*(temp2-1.0d0)
                case (4)
                    osolve%temp(i) = 1.d0+x1/yfix_xedge*(temp2-1.0d0)
                case (5)
                    osolve%temp(i) = 1.d0+distz0plane/rx*(temp2-1.0d0)
                case default
                    osolve%temp(i)=1.d0                            
            end select
        !transition zone radially
        else
            !start value at inner edge of transition zone 
            distrot = sqrt(xminedge**2+zmain**2)
            theta = asin(xminedge/distrot)
            ustart = distrot*omega*cos(theta)
            uend = vback
            wstart = -xminedge*omega
            wend = 0.d0
            !setting velocities
            osolve%u(i)=ustart+distzonerad*(uend-ustart)
            osolve%w(i)=wstart+distzonerad*(wend-wstart)
            !setting temperature
            select case(tempcase)
                case (-1)
                    continue
                case (1)
                    osolve%temp(i) = temp2+x1*(1.0d0-temp2)
                case (2)
                    tempstart = temp2
                    tempend = 1.d0
                    osolve%temp(i)=tempstart+distzonerad*(tempend-tempstart)
                case (3)
                    tempstart = 1.d0+xminedge/rx*(temp2-1.0d0)
                    tempend = 1.d0
                    osolve%temp(i)=tempstart+distzonerad*(tempend-tempstart)
                case (4)
                    yfix_xedge = sqrt(max(0.d0,rmain**2-((yminedge-y0)*rx/(ry))**2-zmain**2))
                    tempstart = 1.d0+xminedge/yfix_xedge*(temp2-1.0d0)
                    tempend = 1.d0
                    osolve%temp(i)=tempstart+distzonerad*(tempend-tempstart)
                case (5)
                    distz0plane = sqrt(xedge**2+((yedge-y0)*rx/(ry))**2)
                    tempstart = 1.d0+distz0plane/rx*(temp2-1.0d0)
                    tempend = 1.d0
                    osolve%temp(i)=tempstart+distzonerad*(tempend-tempstart)
                case default
                    osolve%temp(i)=1.d0                            
            end select 
        end if      
    end if

    !x=1 face with influx vback
    if (osolve%x(i)>1.d0-eps) then
        osolve%kfix((i-1)*3+1)=1 ; osolve%u(i)=vback
        
        if (strictflow == 1) then
            osolve%kfix((i-1)*3+2)=1 ; osolve%v(i)=0.d0
            osolve%kfix((i-1)*3+3)=1 ; osolve%w(i)=0.d0
        end if
    end if
    
    !z=1 face 
    if (osolve%z(i) > 1.d0-eps) then
        osolve%kfix((i-1)*3+1)=1 ; osolve%u(i)=0.d0 !x-direction
        osolve%kfix((i-1)*3+2)=1 ; osolve%v(i)=0.d0 !y-direction
        osolve%kfix((i-1)*3+3)=1 ; osolve%w(i)=0.d0 !z-direction
        osolve%kfixt(i)=1 ; osolve%temp(i)=0.d0
    end if
    
    if (.not.vo%influid(i)) then
       osolve%kfixt(i)=1
       osolve%temp(i)=0.d0
     endif

end do

end 
!------------------------------------------------------------------------------|
